"""A liveness prober dag for monitoring composer.googleapis.com/environment/healthy."""
import airflow
from airflow import DAG
from airflow.operators.bash_operator import BashOperator
from datetime import timedelta

default_args = {
    'start_date': airflow.utils.dates.days_ago(0),
    'retries': 1,
    'retry_delay': timedelta(minutes=5)
}

dag = DAG(
    'airflow_monitoring',
    default_args=default_args,
    description='liveness monitoring dag',
    schedule_interval=None,
    dagrun_timeout=timedelta(minutes=20))

# priority_weight has type int in Airflow DB, uses the maximum.
t1 = BashOperator(
    task_id='echo',
    bash_command='echo test',
    dag=dag,
    depends_on_past=False,
    priority_weight=2**31-1)



# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Demonstrates how to use connections in an Airflow DAG."""

import datetime

from airflow import models
from airflow.providers.google.cloud.operators import bigquery
from airflow.providers.google.cloud.operators.bigquery import BigQueryCheckOperator


yesterday = datetime.datetime.combine(
    datetime.datetime.today() - datetime.timedelta(1),
    datetime.datetime.min.time())

default_dag_args = {
    # Setting start date as yesterday starts the DAG immediately when it is
    # detected in the Cloud Storage bucket.
    'start_date': yesterday,
    'retries': 1,
}

# Define a DAG (directed acyclic graph) of tasks.
# Any task you create within the context manager is automatically added to the
# DAG object.
with models.DAG('create_test_2',
        schedule_interval=datetime.timedelta(days=1),
        default_args=default_dag_args) as dag:
    # [START composer_connections_default]
    task_default = bigquery.BigQueryInsertJobOperator(
        task_id='task_default_connection',
        configuration={
            "query": {
                "query": 'SELECT 1',
                "useLegacySql": False
            }
        }
    )
    # [END composer_connections_default]
    # [START composer_connections_explicit]
    # Composer creates a 'google_cloud_default' connection by default.
    task_explicit = bigquery.BigQueryInsertJobOperator(
        task_id='task_explicit_connection',
        gcp_conn_id='google_cloud_default',
        configuration={
            "query": {
                "query": 'SELECT 1',
                "useLegacySql": False
            }
        }
    )
    # [END composer_connections_explicit]
    #

    check0 = BigQueryCheckOperator(task_id='check_public_event_data_exists',
                           sql="""
                            select count(*) > 0
                            from bigquery-public-data.austin_bikeshare.bikeshare_trips
                        """,
                           use_legacy_sql=False)


    check1 = bigquery.BigQueryInsertJobOperator(
        task_id='check_with_insert_operator',
        gcp_conn_id='google_cloud_default',
        configuration={
            "query": {
                "query": 'select distinct 1 > 0 from churn-1.socialnet7.event',
                "useLegacySql": False
            }
        }
    )

    create_table = bigquery.BigQueryCreateEmptyTableOperator(
        task_id="create_table",
        dataset_id='churn-1',
        table_id="metrics",
        schema_fields=[
            {"name": "account_id", "type": "INTEGER", "mode": "REQUIRED"},
            {"name": "metric_time", "type": "TIMESTAMP", "mode": "REQUIRED"},
            {"name": "metric_name", "type": "STRING", "mode": "REQUIRED"},
            {"name": "metric_value", "type": "FLOAT", "mode": "REQUIRED"},
        ],
    )

    # upsert_table = bigquery.BigQueryUpsertTableOperator(
    #     task_id="upsert_table",
    #     dataset_id='churn-1',
    #     table_resource={
    #         "tableReference": {"tableId": "test_table_id"},
    #         "expirationTime": (int(time.time()) + 300) * 1000,
    #     },
    # )

task_default >> task_explicit >> check0 >> check1 >> create_table



from datetime import timedelta, datetime
from airflow import DAG
from airflow.contrib.operators.bigquery_operator import BigQueryOperator
from airflow.contrib.operators.bigquery_check_operator import BigQueryCheckOperator

dag_args = {
    'owner': 'carl.gold',
    'depends_on_past': False,
    'start_date': datetime(2020, 5, 31),
    'end_date': datetime(2020, 6, 1),
    'email': ['carl.steven.gold@gmail.com'],
    'email_on_failure': True,
    'email_on_retry': True,
    'retries': 0,
    'retry_delay': timedelta(minutes=3)}

dag = DAG(
    dag_id='metric_data_test_5',
    start_date=datetime(2020, 5, 31),
    end_date=datetime(2020, 6, 1),
    default_args=dag_args,
    schedule_interval='0 9 * * *')

t1 = BigQueryCheckOperator(task_id='check_event_data_exists',
                           sql="""
                            select count(*) > 0
                            from `fight-churn-with-gcp.churn_1.event`
                            where DATE(event_time) = '2020-05-31'
                        """,
                           use_legacy_sql=False)

t2 = BigQueryOperator(
    task_id='calculate_metric',
    use_legacy_sql=False,
    create_disposition='CREATE_IF_NEEDED',
    write_disposition='WRITE_TRUNCATE',
    allow_large_results=True,
    bigquery_conn_id='bigquery_default',
    time_partitioning={
        "type": 'DAY'
        },
    sql='''
    #standardSQL
        select date('2020-05-31') as date, account_id,  'n_like' as metric, count(*) as value
        from `fight-churn-with-gcp.churn_1.event` e 
        inner join `fight-churn-with-gcp.churn_1.event_type` t on t.event_type_id=e.event_type_id
        where t.event_type_name='like'
          e.event_time <= '2020-05-31'
          and DATE_DIFF('2020-05-31', e.event_time , DAY) < 28
        group by account_id
    ''',
    destination_dataset_table='fight-churn-with-gcp.churn_1.metric_test',
    dag=dag)

t1 >> t2



"""Testing basic BigQuery connections and the churn dataset."""

import datetime

from airflow import models
from airflow.providers.google.cloud.operators import bigquery
from airflow.providers.google.cloud.operators.bigquery import BigQueryCheckOperator, BigQueryUpsertTableOperator, BigQueryInsertJobOperator

project_id = 'churn-test-353716'
dataset_name = 'socialnet7'

yesterday = datetime.datetime.combine(
    datetime.datetime.today() - datetime.timedelta(1),
    datetime.datetime.min.time())

default_dag_args = {
    # Setting start date as yesterday starts the DAG immediately when it is detected in the Cloud Storage bucket.
    'start_date': yesterday,
    'retries': 1,
}


with models.DAG('event_count_table_one_day_4',
        schedule_interval=datetime.timedelta(days=1),
        default_args=default_dag_args) as dag:

    # Checking the churn data & Connection
    check_private = BigQueryCheckOperator(task_id='check_churn_event_data',
                                          sql=f"""
                            select count(*) > 0
                            from {project_id}.{dataset_name}.event
                        """,
                                          location="us-west1",
                                          use_legacy_sql=False)

    event_count_table = BigQueryUpsertTableOperator(
        task_id="upsert_table",
        project_id = project_id,
        dataset_id=dataset_name,
        location="us-west1",
        table_resource={
            "tableReference": {"tableId": "events_per_day"},
            "schema" : {
                "fields" : [
                    {
                        "name" : "event_name",
                        "type" : "string"
                    },
                    {
                        "name" : "event_date",
                        "type" : "date"
                    },
                    {
                        "name" : "event_count",
                        "type" : "integer"
                    }
                ]
            }
        },
    )

    insert_query_job = BigQueryInsertJobOperator(
        task_id="insert_event_count",
        configuration={
            "query": {
                "query": f"""
                    INSERT INTO `{project_id}.{dataset_name}.events_per_day`
                    select event_type_name as event_name, date(event_time) as event_date, count(*) as event_count
                    from `{project_id}.{dataset_name}.event` e 
                    inner join `{project_id}.{dataset_name}.event_type` t
                    on e.event_type_id = t.event_type_id
                    where date(event_time)='{{{{  ds }}}}'
                    group by event_type_name, event_date
                    order by event_date, event_name;
                """,
                "useLegacySql": False,
            }
        },
        location='us-west1',
    )


check_private >> event_count_table >> insert_query_job
